part of 'curtain_module_bloc.dart';

sealed class CurtainModuleState extends Equatable {
  const CurtainModuleState();

  @override
  List<Object> get props => [];
}

class CurtainModuleInitial extends CurtainModuleState {}

class CurtainModuleLoading extends CurtainModuleState {}

class CurtainModuleError extends CurtainModuleState {
  final String message;
  const CurtainModuleError({required this.message});

  @override
  List<Object> get props => [message];
}

class CurtainModuleStatusLoaded extends CurtainModuleState {
  final CurtainModuleStatusModel curtainModuleStatus;

  const CurtainModuleStatusLoaded({required this.curtainModuleStatus});

  @override
  List<Object> get props => [curtainModuleStatus];
}
class CurtainModuleStatusUpdated extends CurtainModuleState {
  final CurtainModuleStatusModel curtainModuleStatus;

  const CurtainModuleStatusUpdated({required this.curtainModuleStatus});

  @override
  List<Object> get props => [curtainModuleStatus];
}
